var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __rest = (this && this.__rest) || function (s, e) {
    var t = {};
    for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p) && e.indexOf(p) < 0)
        t[p] = s[p];
    if (s != null && typeof Object.getOwnPropertySymbols === "function")
        for (var i = 0, p = Object.getOwnPropertySymbols(s); i < p.length; i++) {
            if (e.indexOf(p[i]) < 0 && Object.prototype.propertyIsEnumerable.call(s, p[i]))
                t[p[i]] = s[p[i]];
        }
    return t;
};
import { Connector } from '@web3-react/types';
export class NoWalletLinkError extends Error {
    constructor() {
        super('WalletLink not installed');
        this.name = NoWalletLinkError.name;
        Object.setPrototypeOf(this, NoWalletLinkError.prototype);
    }
}
function parseChainId(chainId) {
    return Number.parseInt(chainId, 16);
}
export class WalletLink extends Connector {
    constructor(actions, options, connectEagerly = true) {
        super(actions);
        this.options = options;
        if (connectEagerly) {
            this.providerPromise = this.startListening(connectEagerly);
        }
    }
    startListening(connectEagerly) {
        var _a;
        return __awaiter(this, void 0, void 0, function* () {
            const _b = this.options, { url } = _b, options = __rest(_b, ["url"]);
            const provider = yield import('walletlink')
                .then((m) => m.WalletLink)
                .then((WalletLink) => new WalletLink(options).makeWeb3Provider(url));
            this.provider = (_a = provider) !== null && _a !== void 0 ? _a : undefined;
            if (this.provider) {
                this.provider.on('connect', ({ chainId }) => {
                    this.actions.update({ chainId: parseChainId(chainId) });
                });
                this.provider.on('disconnect', (error) => {
                    this.actions.reportError(error);
                });
                this.provider.on('chainChanged', (chainId) => {
                    this.actions.update({ chainId: parseChainId(chainId) });
                });
                this.provider.on('accountsChanged', (accounts) => {
                    this.actions.update({ accounts });
                });
                if (connectEagerly) {
                    return Promise.all([
                        this.provider.request({ method: 'eth_chainId' }),
                        this.provider.request({ method: 'eth_accounts' }),
                    ])
                        .then(([chainId, accounts]) => {
                        if (accounts.length > 0) {
                            this.actions.update({ chainId: parseChainId(chainId), accounts });
                        }
                    })
                        .catch((error) => {
                        console.debug('Could not connect eagerly', error);
                    });
                }
            }
        });
    }
    activate() {
        return __awaiter(this, void 0, void 0, function* () {
            this.actions.startActivation();
            if (!this.providerPromise) {
                this.providerPromise = this.startListening(false);
            }
            yield this.providerPromise;
            if (this.provider) {
                yield Promise.all([
                    this.provider.request({ method: 'eth_chainId' }),
                    this.provider.request({ method: 'eth_requestAccounts' }),
                ])
                    .then(([chainId, accounts]) => {
                    this.actions.update({ chainId: Number.parseInt(chainId, 16), accounts });
                })
                    .catch((error) => {
                    this.actions.reportError(error);
                });
            }
            else {
                this.actions.reportError(new NoWalletLinkError());
            }
        });
    }
}
